import schedule
import threading
import time
from real_trader import RealTrader
from flask import Flask, request, abort, jsonify


def run_daily(time_str, job_func):
    schedule.every().monday.at(time_str).do(job_func)
    schedule.every().tuesday.at(time_str).do(job_func)
    schedule.every().wednesday.at(time_str).do(job_func)
    schedule.every().thursday.at(time_str).do(job_func)
    schedule.every().friday.at(time_str).do(job_func)


def run_pending_tasks():
    while True:
        schedule.run_pending()
        time.sleep(1)


# Run the task scheduler in a new thread
threading.Thread(target=run_pending_tasks).start()
trader = RealTrader()

### test code start
trader.login()
# time.sleep(5)
# trader.logout()
### test code end

# 打开并登陆交易软件
run_daily("09:00", lambda: trader.login())
# 激活交易软件
run_daily("09:15", lambda: trader.get_balance())
# 关闭交易软件
run_daily("15:30", lambda: trader.logout())


app = Flask(__name__)


@app.route("/yu/healthcheck", methods=["GET"])
def health_check():
    return "ok", 200


@app.route("/yu/buy", methods=["POST"])
def buy():
    """Buy an item with given parameters."""
    # Get data from request body
    data = request.get_json()
    code = data.get("code")
    price_str = data.get("price")
    amount_str = data.get("amount")

    try:
        price = float(price_str)
        amount = int(amount_str)
        print(f"buy: {code}, {price}, {amount}")
        result = trader.buy(code, price, amount)

        response = {"success": True, "data": result}
        return jsonify(response), 200
    except ValueError:
        abort(
            400,
            description="Invalid format for one or more of the following args: code, price, amount.",
        )


@app.route("/yu/sell", methods=["POST"])
def sell():
    """Sell an item with given parameters."""
    # Get data from request body
    data = request.get_json()
    code = data.get("code")
    price_str = data.get("price")
    amount_str = data.get("amount")

    try:
        price = float(price_str)
        amount = int(amount_str)
        print(f"sell: {code}, {price}, {amount}")
        result = trader.sell(code, price, amount)

        response = {"success": True, "data": result}
        return jsonify(response), 200
    except ValueError:
        abort(
            400,
            description="Invalid format for one or more of the following args: code, price, amount.",
        )


@app.route("/yu/cancel/<entrust_no>", methods=["DELETE"])
def cancel(entrust_no):
    print(f"cancel: {entrust_no}")
    result = trader.cancel(entrust_no)

    response = {"success": True, "data": result}
    return jsonify(response), 200


@app.route("/yu/balance", methods=["GET"])
def get_balance():
    """Get the balance of the account."""
    balance = trader.get_balance()
    print(f"balance: {balance}")

    response = {"success": True, "data": balance}
    return jsonify(response), 200


@app.route("/yu/positions", methods=["GET"])
def get_positions():
    """Get the positions of the account."""
    positions = trader.get_positions()

    response = {"success": True, "data": positions}
    return jsonify(response), 200


@app.route("/yu/todaytrades", methods=["GET"])
def get_today_trades():
    """Get the today's trades of the account."""
    trades = trader.get_today_trades()

    response = {"success": True, "data": trades}
    return jsonify(response), 200


@app.route("/yu/todayentrust", methods=["GET"])
def get_today_entrust():
    """Get the today's entrust of the account."""
    entrust = trader.get_today_entrust()

    response = {"success": True, "data": entrust}
    return jsonify(response), 200


@app.route("/yu/refresh", methods=["GET"])
def refresh():
    """Refresh the account."""
    trader.refresh()

    response = {"success": True, "data": "Account data refreshed successfully."}
    return jsonify(response), 200


if __name__ == "__main__":
    PORT = 9527
    print(f"Server listening on port {PORT}")
    app.run(debug=False, host="0.0.0.0", port=PORT)
